﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Drawing.Printing;
using System.Printing;

namespace TestWpfPrint
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            PrintDialog pd = new PrintDialog();
            textBox1.Text = pd.PrintQueue.FullName;
        }

        private void buttonExit_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private void buttonWithDialog_Click(object sender, RoutedEventArgs e)
        {
            Print(textBox1.Text, true);
        }

        private void buttonWithoutDialog_Click(object sender, RoutedEventArgs e)
        {
            Print(textBox1.Text, false);
        }

        private void Print(string printerName, bool showPrintDialog)
        {
            var printDialog = new PrintDialog();
            printDialog.UserPageRangeEnabled = true;
            printDialog.MinPage = 1;
            printDialog.MaxPage = 1;
            printDialog.PageRange = new PageRange(1, 1);

            #region Select Printer Name
            if (!string.IsNullOrEmpty(printerName))
            {
                PrintServer server = null;
                string partServerName = null;
                string partPrinterName = null;

                partPrinterName = printerName;
                if (printerName.StartsWith(@"\\") && !printerName.Contains(@"http://"))
                {
                    int posPrinterName = printerName.IndexOf('\\', 2);
                    if (posPrinterName != -1)
                    {
                        partServerName = printerName.Substring(0, posPrinterName);
                        partPrinterName = printerName.Substring(posPrinterName + 1);
                        server = new PrintServer(partServerName);
                    }
                }
                if (server == null) server = new LocalPrintServer();
                printDialog.PrintQueue = server.GetPrintQueue(partPrinterName);
            }
            #endregion

            bool needPrint = true;

            if (showPrintDialog)
            {
                needPrint = printDialog.ShowDialog().GetValueOrDefault(true);
                printerName = printDialog.PrintQueue.FullName;
            }

            if (needPrint)
            {
                double printableAreaWidth = printDialog.PrintableAreaWidth;
                double printableAreaHeight = printDialog.PrintableAreaHeight;

                var document = new StiWpfDocumentPaginator(printerName, printableAreaWidth, printableAreaHeight);

                printDialog.PrintDocument(document, "Test print Wpf");
            }

        }
     
    }


    public class StiWpfDocumentPaginator : DocumentPaginator
    {
        #region Fields
        private string printerName;
        private double printableWidth;
        private double printableHeight;
        #endregion

        #region Properties
        public override bool IsPageCountValid
        {
            get
            {
                return true;
            }
        }

        private int pageCount;
        public override int PageCount
        {
            get
            {
                return pageCount;
            }
        }

        public override Size PageSize
        {
            get
            {
                return Size.Empty;
            }
            set
            {
            }
        }

        public override IDocumentPaginatorSource Source
        {
            get
            {
                return null;
            }
        }
        #endregion

        #region Methods
        public override DocumentPage GetPage(int pageNumber)
        {
            DrawingVisual visual = new DrawingVisual();
            DrawingContext dc = visual.RenderOpen();

            Pen pen = new Pen(new SolidColorBrush(Colors.Red), 3);
            dc.DrawRectangle(Brushes.Aqua, pen, new Rect(0, 0, 600, 200));
            dc.DrawText(new FormattedText("Test print Wpf", System.Globalization.CultureInfo.CurrentCulture, FlowDirection.LeftToRight, new Typeface("Arial"), 32, Brushes.Brown), new Point(10, 10));
            dc.DrawText(new FormattedText("PrinterName: " + printerName, System.Globalization.CultureInfo.CurrentCulture, FlowDirection.LeftToRight, new Typeface("Arial"), 24, Brushes.Black), new Point(10, 60));
            dc.DrawText(new FormattedText(string.Format("Printable area: {0:F2} x {1:F2}", printableWidth, printableHeight), System.Globalization.CultureInfo.CurrentCulture, FlowDirection.LeftToRight, new Typeface("Arial"), 24, Brushes.Black), new Point(10, 90));
 
            dc.Close();
            return new DocumentPage(visual);
        }
        #endregion

        public StiWpfDocumentPaginator(string printerName, double printableWidth, double printableHeight)
        {
            this.pageCount = 1;
            this.printerName = printerName;
            this.printableWidth = printableWidth;
            this.printableHeight = printableHeight;
        }
    }


}


